@@ -2,7 +2,7 @@
 
 # py_tools_ds
 #
-# Copyright (C) 2019  Daniel Scheffler (GFZ Potsdam, daniel.scheffler@gfz-potsdam.de)
+# Copyright (C) 2016-2021  Daniel Scheffler (GFZ Potsdam, daniel.scheffler@gfz-potsdam.de)
 #
 # This software was developed within the context of the GeoMultiSens project funded
 # by the German Federal Ministry of Education and Research
@@ -23,14 +23,12 @@
 
 __author__ = "Daniel Scheffler"
 
-from six import PY3
-
 from shapely.wkb import loads
 from osgeo import gdal, osr, ogr
 
 from ...io.raster.gdal import get_GDAL_ds_inmem
 from ...processing.progress_mon import ProgressBar, Timer
-from ...compatibility.python.exceptions import TimeoutError as TimeoutError_comp
+from ..raster.reproject import warp_ndarray
 
 
 def raster2polygon(array, gt, prj, DN2extract=1, exact=True, maxfeatCount=None,
@@ -53,16 +51,13 @@ def raster2polygon(array, gt, prj, DN2extract=1, exact=True, maxfeatCount=None,
 
     # downsample input array in case is has more than 1e8 pixels to prevent crash
     if not exact and array.size > 1e8:  # 10000 x 10000 px
-        zoom_factor = 1e8 / array.size
-
         # downsample with nearest neighbour
-        from skimage.transform import rescale  # import here to avoid static TLS import error
-        array = rescale(array, zoom_factor, order=0, preserve_range=True, mode='edge').astype(bool)
-
-        # update pixel sizes within gt
-        gt = list(gt)
-        gt[1] /= zoom_factor
-        gt[5] /= zoom_factor
+        zoom_factor = (8000 * 8000 / array.size) ** 0.5
+        array, gt, prj = warp_ndarray(array, gt, prj,
+                                      out_gsd=(gt[1] / zoom_factor,
+                                               gt[5] / zoom_factor),
+                                      rspAlg='near',
+                                      q=True)
 
     src_ds = get_GDAL_ds_inmem(array, gt, prj)
     src_band = src_ds.GetRasterBand(1)
@@ -80,19 +75,31 @@ def raster2polygon(array, gt, prj, DN2extract=1, exact=True, maxfeatCount=None,
     mem_layer.CreateField(fd)
 
     # set callback
-    callback = ProgressBar(prefix='Polygonize progress    ', suffix='Complete', barLength=50, timeout=timeout,
-                           use_as_callback=True) \
+    callback = \
+        ProgressBar(prefix='Polygonize progress    ',
+                    suffix='Complete',
+                    barLength=50,
+                    timeout=timeout,
+                    use_as_callback=True) \
         if progress and not q else Timer(timeout, use_as_callback=True) if timeout else None
 
     # run the algorithm
-    status = gdal.Polygonize(src_band, src_band.GetMaskBand(), mem_layer, 0, ["8CONNECTED=8"] if exact else [],
+    status = gdal.Polygonize(src_band,
+                             src_band.GetMaskBand(),
+                             mem_layer,
+                             0,
+                             ["8CONNECTED=8"] if exact else [],
                              callback=callback)
 
     # handle exit status other than 0 (fail)
     if status != 0:
         errMsg = gdal.GetLastErrorMsg()
+
+        # Catch the KeyboardInterrupt raised in case of a timeout within the callback. It seems like there is no other
+        # way to catch exceptions within callbacks.
         if errMsg == 'User terminated':
-            raise TimeoutError('raster2polygon timed out!') if PY3 else TimeoutError_comp('raster2polygon timed out!')
+            raise TimeoutError('raster2polygon timed out!')
+
         raise Exception(errMsg)
 
     # extract polygon
@@ -130,7 +137,7 @@ def raster2polygon(array, gt, prj, DN2extract=1, exact=True, maxfeatCount=None,
             GDF.loc[i] = [loads(element.GetGeometryRef().ExportToWkb()).buffer(0), DN2extract]
             del element
         else:
-            raise TimeoutError('raster2polygon timed out!') if PY3 else TimeoutError_comp('raster2polygon timed out!')
+            raise TimeoutError('raster2polygon timed out!')
 
     GDF = GDF.dissolve(by='DN')
 
